'''
Copyright (C) 2021  Alexander "PapaTutuWawa"

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
'''
import importlib
import asyncio

import aioxmpp
import toml

from mira.subscription import SubscriptionManager

def message_wrapper(to, body):
    msg = aioxmpp.Message(
        type_=aioxmpp.MessageType.CHAT,
        to=to)
    msg.body[None] = body
    return msg

class MiraBot:
    def __init__(self):
        # Bot specific settings
        self._jid = ""
        self._password = ""
        self._client = None
        self._avatar = None

        self._modules = {} # Module name -> module
        self._subscription_manager = SubscriptionManager()

    def load_config(self, path):
        data = toml.load(path)

        self._jid = aioxmpp.JID.fromstr(data['jid'])
        self._password = data['password']
        self._avatar = data.get('avatar', None)

        for module in data['modules']:
            mod = importlib.import_module(module['name'])
            self._modules[mod.NAME] = mod.get_instance(self, module)
            self._modules[mod.NAME].set_name(mod.NAME)

    async def connect(self):
        self._client = aioxmpp.PresenceManagedClient(
            self._jid,
            aioxmpp.make_security_layer(self._password))
        async with self._client.connected():
            self._client.stream.register_message_callback(
                aioxmpp.MessageType.CHAT,
                None,
                self._on_message)

            if self._avatar:
                with open(self._avatar, 'rb') as avatar_file:
                    data = avatar_file.read()
                avatar_set = aioxmpp.avatar.AvatarSet()
                # TODO: Detect MIME type
                avatar_set.add_avatar_image('image/png', image_bytes=data)
                avatar = self._client.summon(aioxmpp.avatar.AvatarService)
                await avatar.publish_avatar_set(avatar_set)

            while True:
                await asyncio.sleep(1)

    def _on_message(self, message):
        # Automatically handles sending a message receipt and dealing
        # with unwanted messages
        if (message.type_ != aioxmpp.MessageType.CHAT or
            not message.body):
            return

        cmd = str(message.body.any()).split(' ')

        receipt = aioxmpp.mdr.compose_receipt(message)
        self._client.enqueue(receipt)

        if not cmd[0] in self._modules:
            self._client.enqueue(message_wrapper(message.from_, "Unbekannter Befehl"))
            return

        self._modules[cmd[0]]._on_command(cmd[1:], message)

    # Module Function: Send message
    def send_message(self, message):
        self._client.enqueue(message)

    # Module Function: Send a message to @to with @body
    def send_message_wrapper(self, to, body):
        self.send_message(message_wrapper(to, body))

    # Module Function
    def get_subscription_manager(self):
        return self._subscription_manager
        
def main():
    bot = MiraBot()
    bot.load_config("./config.toml")

    loop = asyncio.get_event_loop()
    loop.run_until_complete(bot.connect())
    loop.close()
